#ifndef MODEL_DATA_H_
#define MODEL_DATA_H_

extern const unsigned char g_model_data[];
extern const int g_model_data_len;
extern const int g_model_dims[];
extern const int g_model_dims_len;

#endif  // MODEL_DATA_H_
