from .test_cnn1d import TestModelsCNN1D
from .test_cnn2d import TestModelsCNN2D
from .test_mlp import TestModelsMLP
from .test_quantizedresnet1d import TestModelsQuantizedResnet1D
from .test_quantizedresnet2d import TestModelsQuantizedResnet2D
from .test_resnet1d import TestModelResNet1D
from .test_resnet2d import TestModelsResNet2D
