/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.3.9.1 at Thu Sep 12 09:14:47 2019. */

#include "stm32msg.pb.h"

/* @@protoc_insertion_point(includes) */
#if PB_PROTO_HEADER_VERSION != 30
#error Regenerate this file with the current version of nanopb generator.
#endif

const EnumCmd reqMsg_cmd_default = EnumCmd_CMD_SYS_INFO;
const uint32_t aiBufferShapeMsg_format_default = 0u;
const uint32_t aiBufferShapeMsg_n_batches_default = 1u;
const uint32_t aiBufferShapeMsg_height_default = 1u;
const uint32_t aiBufferShapeMsg_width_default = 1u;
const uint32_t aiBufferShapeMsg_channels_default = 1u;
const float aiBufferShapeMsg_scale_default = 0;
const int32_t aiBufferShapeMsg_zeropoint_default = 0;


const pb_field_t reqMsg_fields[6] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, reqMsg, reqid, reqid, 0),
    PB_FIELD(  2, UENUM   , REQUIRED, STATIC  , OTHER, reqMsg, cmd, reqid, &reqMsg_cmd_default),
    PB_FIELD(  3, UINT32  , REQUIRED, STATIC  , OTHER, reqMsg, param, cmd, 0),
    PB_FIELD(  4, STRING  , REQUIRED, STATIC  , OTHER, reqMsg, name, param, 0),
    PB_FIELD(  5, UINT32  , REQUIRED, STATIC  , OTHER, reqMsg, opt, name, 0),
    PB_LAST_FIELD
};

const pb_field_t aiRunReportMsg_fields[6] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, aiRunReportMsg, id, id, 0),
    PB_FIELD(  2, UINT32  , REQUIRED, STATIC  , OTHER, aiRunReportMsg, signature, id, 0),
    PB_FIELD(  3, UINT32  , REQUIRED, STATIC  , OTHER, aiRunReportMsg, num_inferences, signature, 0),
    PB_FIELD(  4, UINT32  , REQUIRED, STATIC  , OTHER, aiRunReportMsg, n_nodes, num_inferences, 0),
    PB_FIELD(  5, FLOAT   , REQUIRED, STATIC  , OTHER, aiRunReportMsg, elapsed_ms, n_nodes, 0),
    PB_LAST_FIELD
};

const pb_field_t aiBufferShapeMsg_fields[8] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, aiBufferShapeMsg, format, format, &aiBufferShapeMsg_format_default),
    PB_FIELD(  2, UINT32  , REQUIRED, STATIC  , OTHER, aiBufferShapeMsg, n_batches, format, &aiBufferShapeMsg_n_batches_default),
    PB_FIELD(  3, UINT32  , REQUIRED, STATIC  , OTHER, aiBufferShapeMsg, height, n_batches, &aiBufferShapeMsg_height_default),
    PB_FIELD(  4, UINT32  , REQUIRED, STATIC  , OTHER, aiBufferShapeMsg, width, height, &aiBufferShapeMsg_width_default),
    PB_FIELD(  5, UINT32  , REQUIRED, STATIC  , OTHER, aiBufferShapeMsg, channels, width, &aiBufferShapeMsg_channels_default),
    PB_FIELD(  6, FLOAT   , REQUIRED, STATIC  , OTHER, aiBufferShapeMsg, scale, channels, &aiBufferShapeMsg_scale_default),
    PB_FIELD(  7, INT32   , REQUIRED, STATIC  , OTHER, aiBufferShapeMsg, zeropoint, scale, &aiBufferShapeMsg_zeropoint_default),
    PB_LAST_FIELD
};

const pb_field_t aiNetworkInfoMsg_fields[21] = {
    PB_FIELD(  1, STRING  , REQUIRED, STATIC  , FIRST, aiNetworkInfoMsg, model_name, model_name, 0),
    PB_FIELD(  2, STRING  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, model_signature, model_name, 0),
    PB_FIELD(  3, STRING  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, model_datetime, model_signature, 0),
    PB_FIELD(  4, STRING  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, compile_datetime, model_datetime, 0),
    PB_FIELD(  5, STRING  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, runtime_revision, compile_datetime, 0),
    PB_FIELD(  6, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, runtime_version, runtime_revision, 0),
    PB_FIELD(  7, STRING  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, tool_revision, runtime_version, 0),
    PB_FIELD(  8, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, tool_version, tool_revision, 0),
    PB_FIELD(  9, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, tool_api_version, tool_version, 0),
    PB_FIELD( 10, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, api_version, tool_api_version, 0),
    PB_FIELD( 11, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, interface_api_version, api_version, 0),
    PB_FIELD( 12, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, n_macc, interface_api_version, 0),
    PB_FIELD( 13, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, n_inputs, n_macc, 0),
    PB_FIELD( 14, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, n_outputs, n_inputs, 0),
    PB_FIELD( 15, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, n_nodes, n_outputs, 0),
    PB_FIELD( 16, MESSAGE , REPEATED, CALLBACK, OTHER, aiNetworkInfoMsg, inputs, n_nodes, &aiBufferShapeMsg_fields),
    PB_FIELD( 17, MESSAGE , REPEATED, CALLBACK, OTHER, aiNetworkInfoMsg, outputs, inputs, &aiBufferShapeMsg_fields),
    PB_FIELD( 18, MESSAGE , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, activations, outputs, &aiBufferShapeMsg_fields),
    PB_FIELD( 19, MESSAGE , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, weights, activations, &aiBufferShapeMsg_fields),
    PB_FIELD( 20, UINT32  , REQUIRED, STATIC  , OTHER, aiNetworkInfoMsg, signature, weights, 0),
    PB_LAST_FIELD
};

const pb_field_t aiBufferByteMsg_fields[3] = {
    PB_FIELD(  1, MESSAGE , REQUIRED, STATIC  , FIRST, aiBufferByteMsg, shape, shape, &aiBufferShapeMsg_fields),
    PB_FIELD(  2, BYTES   , REQUIRED, CALLBACK, OTHER, aiBufferByteMsg, datas, shape, 0),
    PB_LAST_FIELD
};

const pb_field_t syncMsg_fields[3] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, syncMsg, version, version, 0),
    PB_FIELD(  4, UINT32  , REQUIRED, STATIC  , OTHER, syncMsg, capability, version, 0),
    PB_LAST_FIELD
};

const pb_field_t sysinfoMsg_fields[5] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, sysinfoMsg, devid, devid, 0),
    PB_FIELD(  2, UINT32  , REQUIRED, STATIC  , OTHER, sysinfoMsg, sclock, devid, 0),
    PB_FIELD(  3, UINT32  , REQUIRED, STATIC  , OTHER, sysinfoMsg, hclock, sclock, 0),
    PB_FIELD(  4, UINT32  , REQUIRED, STATIC  , OTHER, sysinfoMsg, cache, hclock, 0),
    PB_LAST_FIELD
};

const pb_field_t ackMsg_fields[3] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, ackMsg, param, param, 0),
    PB_FIELD(  2, UENUM   , REQUIRED, STATIC  , OTHER, ackMsg, error, param, 0),
    PB_LAST_FIELD
};

const pb_field_t logMsg_fields[3] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, logMsg, level, level, 0),
    PB_FIELD(  2, STRING  , REQUIRED, STATIC  , OTHER, logMsg, str, level, 0),
    PB_LAST_FIELD
};

const pb_field_t nodeMsg_fields[5] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, nodeMsg, type, type, 0),
    PB_FIELD(  2, UINT32  , REQUIRED, STATIC  , OTHER, nodeMsg, id, type, 0),
    PB_FIELD(  3, FLOAT   , REQUIRED, STATIC  , OTHER, nodeMsg, duration, id, 0),
    PB_FIELD(  4, MESSAGE , REQUIRED, STATIC  , OTHER, nodeMsg, buffer, duration, &aiBufferByteMsg_fields),
    PB_LAST_FIELD
};

const pb_field_t respMsg_fields[10] = {
    PB_FIELD(  1, UINT32  , REQUIRED, STATIC  , FIRST, respMsg, reqid, reqid, 0),
    PB_FIELD(  2, UENUM   , REQUIRED, STATIC  , OTHER, respMsg, state, reqid, 0),
    PB_ONEOF_FIELD(payload,  10, MESSAGE , ONEOF, STATIC  , OTHER, respMsg, sync, state, &syncMsg_fields),
    PB_ONEOF_FIELD(payload,  11, MESSAGE , ONEOF, STATIC  , UNION, respMsg, sinfo, state, &sysinfoMsg_fields),
    PB_ONEOF_FIELD(payload,  12, MESSAGE , ONEOF, STATIC  , UNION, respMsg, ack, state, &ackMsg_fields),
    PB_ONEOF_FIELD(payload,  13, MESSAGE , ONEOF, STATIC  , UNION, respMsg, log, state, &logMsg_fields),
    PB_ONEOF_FIELD(payload,  14, MESSAGE , ONEOF, STATIC  , UNION, respMsg, node, state, &nodeMsg_fields),
    PB_ONEOF_FIELD(payload,  20, MESSAGE , ONEOF, STATIC  , UNION, respMsg, ninfo, state, &aiNetworkInfoMsg_fields),
    PB_ONEOF_FIELD(payload,  21, MESSAGE , ONEOF, STATIC  , UNION, respMsg, report, state, &aiRunReportMsg_fields),
    PB_LAST_FIELD
};










/* Check that field information fits in pb_field_t */
#if !defined(PB_FIELD_32BIT)
/* If you get an error here, it means that you need to define PB_FIELD_32BIT
 * compile-time option. You can do that in pb.h or on compiler command line.
 * 
 * The reason you need to do this is that some of your messages contain tag
 * numbers or field sizes that are larger than what can fit in 8 or 16 bit
 * field descriptors.
 */
PB_STATIC_ASSERT((pb_membersize(aiNetworkInfoMsg, activations) < 65536 && pb_membersize(aiNetworkInfoMsg, weights) < 65536 && pb_membersize(aiBufferByteMsg, shape) < 65536 && pb_membersize(nodeMsg, buffer) < 65536 && pb_membersize(respMsg, payload.sync) < 65536 && pb_membersize(respMsg, payload.sinfo) < 65536 && pb_membersize(respMsg, payload.ack) < 65536 && pb_membersize(respMsg, payload.log) < 65536 && pb_membersize(respMsg, payload.node) < 65536 && pb_membersize(respMsg, payload.ninfo) < 65536 && pb_membersize(respMsg, payload.report) < 65536), YOU_MUST_DEFINE_PB_FIELD_32BIT_FOR_MESSAGES_reqMsg_aiRunReportMsg_aiBufferShapeMsg_aiNetworkInfoMsg_aiBufferByteMsg_syncMsg_sysinfoMsg_ackMsg_logMsg_nodeMsg_respMsg)
#endif

#if !defined(PB_FIELD_16BIT) && !defined(PB_FIELD_32BIT)
/* If you get an error here, it means that you need to define PB_FIELD_16BIT
 * compile-time option. You can do that in pb.h or on compiler command line.
 * 
 * The reason you need to do this is that some of your messages contain tag
 * numbers or field sizes that are larger than what can fit in the default
 * 8 bit descriptors.
 */
PB_STATIC_ASSERT((pb_membersize(aiNetworkInfoMsg, activations) < 256 && pb_membersize(aiNetworkInfoMsg, weights) < 256 && pb_membersize(aiBufferByteMsg, shape) < 256 && pb_membersize(nodeMsg, buffer) < 256 && pb_membersize(respMsg, payload.sync) < 256 && pb_membersize(respMsg, payload.sinfo) < 256 && pb_membersize(respMsg, payload.ack) < 256 && pb_membersize(respMsg, payload.log) < 256 && pb_membersize(respMsg, payload.node) < 256 && pb_membersize(respMsg, payload.ninfo) < 256 && pb_membersize(respMsg, payload.report) < 256), YOU_MUST_DEFINE_PB_FIELD_16BIT_FOR_MESSAGES_reqMsg_aiRunReportMsg_aiBufferShapeMsg_aiNetworkInfoMsg_aiBufferByteMsg_syncMsg_sysinfoMsg_ackMsg_logMsg_nodeMsg_respMsg)
#endif


/* @@protoc_insertion_point(eof) */
