from tensorflow.keras.layers import Conv2D as Conv
from tensorflow.keras.layers import AveragePooling2D as AveragePooling
from tensorflow.keras.layers import MaxPooling2D as MaxPooling
from tensorflow.keras.layers import ZeroPadding2D as ZeroPadding
