from .CIFAR10 import CIFAR10
from .CORe50 import CORe50
from .EllcieHAR import EllcieHAR
from .GSC import GSC
from .GTSRB import GTSRB
from .HD import HD
from .UCI_HAR import UCI_HAR
from .WSMNIST import WSMNIST

__all__ = ['CIFAR10', 'CORe50', 'EllcieHAR', 'GSC', 'GTSRB', 'HD', 'UCI_HAR', 'WSMNIST']
