from .test_qualia_codegen_linux import TestQualiaCodeGenLinux
from .test_qualia_codegen_nucleol452rep import TestQualiaCodeGenNucleoL452REP
from .test_qualia_codegen_sparkfunedge import TestQualiaCodeGenSparkFunEdge
from .test_stm32cubeai_nucleol452rep import TestSTM32CubeAINucleoL452REP
from .test_tflitemicro_sparkfunedge import TestTFLiteMicroSparkFunEdge

__all__ = ['TestQualiaCodeGenLinux', 'TestQualiaCodeGenNucleoL452REP', 'TestQualiaCodeGenSparkFunEdge', 'TestSTM32CubeAINucleoL452REP',
           'TestTFLiteMicroSparkFunEdge']
