from tensorflow.keras.layers import Conv1D as Conv
from tensorflow.keras.layers import AveragePooling1D as AveragePooling
from tensorflow.keras.layers import MaxPooling1D as MaxPooling
from tensorflow.keras.layers import ZeroPadding1D as ZeroPadding
