/**
  ******************************************************************************
  * @file    ai_common_config.h
  * @author  AST Embedded Analytics Research Platform
  * @date    18-May-2018
  * @brief   header file of AI platform common compile configuration defines
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2018 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0044, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0044
  *
  ******************************************************************************
  */
#ifndef __AI_COMMON_CONFIG_H_
#define __AI_COMMON_CONFIG_H_
#pragma once

/*!
 * @defgroup layers Layers Compilation Config Definitions
 * @brief definition 
 *
 */

#define HAS_PROFILE_FLOAT
#define HAS_PROFILE_FIXED


#endif    /*__AI_COMMON_CONFIG_H_*/
